
=head1 NAME

GraphToolQuadrilateral.pl - Add a quadrilateral graph tool and graph object to
the GraphTool.

=head1 DESCRIPTION

GraphTool objects let you provide an interactive graphing tool for students to
enter graphical answers. This macro adds a quadrilateral graph tool and graph
object to the GraphTool.

=head1 USAGE

The syntax for a quadrilateral graph object to pass to the GraphTool constructor
is the name C<"quadrilateral">, followed by the word C<"solid"> or C<"dashed">
to indicate if the quadrilateral is expected to be drawn solid or dashed,
followed by the four vertices of the quadrilateral. For example:

    "{ quadrilateral, solid, (0, 0), (4, 3), (2, 3), (4, -3) }"

To make the quadrilateral tool available in the GraphTool add
C<QuadrilateralTool> to the C<availableTools> GraphTool option as in

    $gt = GraphTool(
        "{ quadrilateral, solid, ($x1, $y1), ($x2, $y2), ($x3, $y3) }"
    )->with(availableTools => [ 'QuadrilateralTool', 'FillTool', 'SolidDashTool' ]);

Note that a quadrilateral graph object supports usage with the C<FillTool> and
C<SolidDashTool> tools.

=cut

BEGIN { strict->import }

sub _GraphToolQuadrilateral_init {
	ADD_CSS_FILE("${main::htmlURL}js/GraphTool/quadrilateral.css", 1);
	ADD_JS_FILE("${main::htmlURL}js/GraphTool/quadrilateral.js", 1, { defer => undef });
}

loadMacros('parserGraphTool.pl');

parser::GraphTool->addGraphObjects(
	quadrilateral => {
		js   => 'graphTool.quadrilateral.graphObject',
		tikz => {
			code => sub {
				my $gt = shift;

				my @points = @{ $_->{data} }[ 2 .. 5 ];
				my ($p1x, $p1y) = @{ $points[0]{data} };
				my ($p2x, $p2y) = @{ $points[1]{data} };
				my ($p3x, $p3y) = @{ $points[2]{data} };
				my ($p4x, $p4y) = @{ $points[3]{data} };

				return (
					"\\draw[thick, blue, line width = 2.5pt, $_->{data}[1]]"
						. " ($p1x, $p1y) -- ($p2x, $p2y) -- ($p3x, $p3y) -- ($p4x, $p4y) -- cycle;",
					[
						"($p1x, $p1y) -- ($p2x, $p2y) -- ($p3x, $p3y) -- ($p4x, $p4y) -- cycle",
						sub {
							my ($x, $y) = @_;

							# Check to see if the point is on the border.
							for my $i (0 .. 3) {
								my ($x1, $y1) = @{ $points[$i]{data} };
								my ($x2, $y2) = @{ $points[ ($i + 1) % 4 ]{data} };
								return 0
									if ($x <= max($x1, $x2)
										&& $x >= min($x1, $x2)
										&& $y <= max($y1, $y2)
										&& $y >= min($y1, $y2)
										&& ($y - $y1) * ($x2 - $x1) - ($y2 - $y1) * ($x - $x1) == 0);
							}

							# Check to see if the point is inside.
							my $isIn = 0;
							for my $i (0 .. 3) {
								my ($x1, $y1) = @{ $points[$i]{data} };
								my ($x2, $y2) = @{ $points[ ($i + 1) % 4 ]{data} };
								if ($y1 > $y != $y2 > $y && $x - $x1 < (($x2 - $x1) * ($y - $y1)) / ($y2 - $y1)) {
									$isIn = !$isIn;
								}
							}
							return 1 if $isIn;

							return -1;
						}
					]
				);
			}
		},
		cmp => sub {
			my $quadrilateral = shift;

			my $solid_dashed = $quadrilateral->{data}[1];
			my @points       = @{ $quadrilateral->{data} }[ 2 .. 5 ];

			return (
				sub {
					my $point = shift;
					my ($x, $y) = $point->value;

					# Check to see if the point is on the border.
					for my $i (0 .. 3) {
						my ($x1, $y1) = @{ $points[$i]{data} };
						my ($x2, $y2) = @{ $points[ ($i + 1) % 4 ]{data} };
						return 0
							if ($x <= max($x1, $x2)
								&& $x >= min($x1, $x2)
								&& $y <= max($y1, $y2)
								&& $y >= min($y1, $y2)
								&& ($y - $y1) * ($x2 - $x1) - ($y2 - $y1) * ($x - $x1) == 0);
					}

					# Check to see if the point is inside.
					my $isIn = 0;
					for my $i (0 .. 3) {
						my ($x1, $y1) = @{ $points[$i]{data} };
						my ($x2, $y2) = @{ $points[ ($i + 1) % 4 ]{data} };
						if ($y1 > $y != $y2 > $y && $x - $x1 < (($x2 - $x1) * ($y - $y1)) / ($y2 - $y1)) {
							$isIn = !$isIn;
						}
					}
					return 1 if $isIn;

					return -1;
				},
				sub {
					my ($other, $fuzzy) = @_;
					return 0
						if $other->{data}[0] ne 'quadrilateral' || (!$fuzzy && $other->{data}[1] ne $solid_dashed);

                    # Check for the four possible cycles that give the same quadrilateral in both directions.
					for my $i (0 .. 3) {
						my $correct = 1;
						for my $j (0 .. 3) {
							if ($points[ ($i + $j) % 4 ] != $other->{data}[ $j + 2 ]) {
								$correct = 0;
								last;
							}
						}
						return 1 if $correct;

						$correct = 1;
						for my $j (0 .. 3) {
							if ($points[ 3 - ($i + $j) % 4 ] != $other->{data}[ $j + 2 ]) {
								$correct = 0;
								last;
							}
						}
						return 1 if $correct;
					}

					return 0;
				}
			);

		}
	}
);

parser::GraphTool->addTools(QuadrilateralTool => 'graphTool.quadrilateral.tool');

1;
