
=head1 NAME

GraphToolTriangle.pl - Add a triangle graph tool and graph object to the
GraphTool.

=head1 DESCRIPTION

GraphTool objects let you provide an interactive graphing tool for students to
enter graphical answers. This macro adds a triangle graph tool and graph object
to the GraphTool.

=head1 USAGE

The syntax for a triangle graph object to pass to the GraphTool constructor is
the name C<"triangle">, followed by the word C<"solid"> or C<"dashed"> to
indicate if the triangle is expected to be drawn solid or dashed, followed by
the three vertices of the triangle. For example:

    "{ triangl, solid, (-1, 2), (1, 0), (3, 3) }"

To make the triangle tool available in the GraphTool add C<TriangleTool> to
the C<availableTools> GraphTool option as in

    $gt = GraphTool(
        "{ triangle, solid, ($x1, $y1), ($x2, $y2), ($x3, $y3) }"
    )->with(availableTools => [ 'TriangleTool', 'FillTool', 'SolidDashTool' ]);

Note that a triangle graph object supports usage with the C<FillTool> and
C<SolidDashTool> tools.

=cut

BEGIN { strict->import }

sub _GraphToolTriangle_init {
	ADD_CSS_FILE("${main::htmlURL}js/GraphTool/triangle.css", 1);
	ADD_JS_FILE("${main::htmlURL}js/GraphTool/triangle.js", 1, { defer => undef });
}

loadMacros('parserGraphTool.pl');

parser::GraphTool->addGraphObjects(
	triangle => {
		js   => 'graphTool.triangle.graphObject',
		tikz => {
			code => sub {
				my $gt = shift;

				my ($p1x, $p1y) = @{ $_->{data}[2]{data} };
				my ($p2x, $p2y) = @{ $_->{data}[3]{data} };
				my ($p3x, $p3y) = @{ $_->{data}[4]{data} };

				return (
					"\\draw[thick, blue, line width = 2.5pt, $_->{data}[1]]"
						. " ($p1x, $p1y) -- ($p2x, $p2y) -- ($p3x, $p3y) -- cycle;",
					[
						"($p1x, $p1y) -- ($p2x, $p2y) -- ($p3x, $p3y) -- cycle",
						sub {
							my $denominator = ($p2y - $p3y) * ($p1x - $p3x) + ($p3x - $p2x) * ($p1y - $p3y);
							my $s =
								(($p2y - $p3y) * ($_[0] - $p3x) + ($p3x - $p2x) * ($_[1] - $p3y)) / $denominator;
							my $t =
								(($p3y - $p1y) * ($_[0] - $p3x) + ($p1x - $p3x) * ($_[1] - $p3y)) / $denominator;
							if ($s >= 0 && $t >= 0 && $s + $t <= 1) {
								return 0 if ($s == 0 || $t == 0 || $s + $t == 1);
								return 1;
							}
							return -1;
						}
					]
				);
			}
		},
		cmp => sub {
			my $triangle = shift;

			my $solid_dashed = $triangle->{data}[1];
			my $p1           = $triangle->{data}[2];
			my $p2           = $triangle->{data}[3];
			my $p3           = $triangle->{data}[4];

			my ($p1x, $p1y) = $p1->value;
			my ($p2x, $p2y) = $p2->value;
			my ($p3x, $p3y) = $p3->value;
			my $denominator = ($p2y - $p3y) * ($p1x - $p3x) + ($p3x - $p2x) * ($p1y - $p3y);

			return (
				sub {
					my $point = shift;
					my ($x, $y) = $point->value;
					my $s = (($p2y - $p3y) * ($x - $p3x) + ($p3x - $p2x) * ($y - $p3y)) / $denominator;
					my $t = (($p3y - $p1y) * ($x - $p3x) + ($p1x - $p3x) * ($y - $p3y)) / $denominator;
					if ($s >= 0 && $t >= 0 && $s + $t <= 1) {
						return 0 if ($s == 0 || $t == 0 || $s + $t == 1);
						return 1;
					}
					return -1;
				},
				sub {
					my ($other, $fuzzy) = @_;
					return 0 if $other->{data}[0] ne 'triangle' || (!$fuzzy && $other->{data}[1] ne $solid_dashed);

					for my $otherPoint (@{ $other->{data} }[ 2, 3, 4 ]) {
						return 0 if !(grep { $_ == $otherPoint } $p1, $p2, $p3);
					}

					return 1;
				}
			);
		}
	}
);

parser::GraphTool->addTools(TriangleTool => 'graphTool.triangle.tool');

1;
